@@ -27,6 +27,19 @@ from policy_sentry.util.file import read_yaml_file
 logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
 
 
+def header_matches(string, table):
+    """checks if the string is found in the table header"""
+    headers = [chomp(str(x)).lower() for x in table.find_all("th")]
+    match_found = False
+    for header in headers:
+        if string in header:
+            match_found = True
+            break
+    if not match_found:
+        return False
+    return True
+
+
 def get_links_from_base_actions_resources_conditions_page():
     """Gets the links from the actions, resources, and conditions keys page, and returns their filenames."""
     html = requests.get(BASE_DOCUMENTATION_URL)
@@ -148,14 +161,17 @@ def create_database(destination_directory, access_level_overrides_file):
         parents=True, exist_ok=True
     )
 
-    schema = []
+    schema = {}
 
     # for filename in ['list_amazonathena.partial.html']:
-    for filename in [
-        f
-        for f in os.listdir(BUNDLED_HTML_DIRECTORY_PATH)
-        if os.path.isfile(os.path.join(BUNDLED_HTML_DIRECTORY_PATH, f))
-    ]:
+    file_list = []
+    for filename in os.listdir(BUNDLED_HTML_DIRECTORY_PATH):
+        if os.path.isfile(os.path.join(BUNDLED_HTML_DIRECTORY_PATH, filename)):
+            if filename not in file_list:
+                file_list.append(filename)
+
+    file_list.sort()
+    for filename in file_list:
         if not filename.startswith("list_"):
             continue
 
@@ -168,8 +184,10 @@ def create_database(destination_directory, access_level_overrides_file):
             # Get service name
             title = main_content.find("h1", class_="topictitle").text
             title = re.sub(
-                ".*Actions, Resources, and Condition Keys for *", "", str(title)
+                ".*Actions, resources, and condition Keys for *", "", str(title),
+                flags=re.IGNORECASE
             )
+
             title = title.replace("</h1>", "")
             service_name = chomp(title)
 
@@ -183,9 +201,9 @@ def create_database(destination_directory, access_level_overrides_file):
             service_schema = {
                 "service_name": service_name,
                 "prefix": prefix,
-                "privileges": [],
-                "resources": [],
-                "conditions": [],
+                "privileges": {},
+                "resources": {},
+                "conditions": {},
             }
 
             access_level_overrides_cfg = get_action_access_level_overrides_from_yml(
@@ -197,9 +215,7 @@ def create_database(destination_directory, access_level_overrides_file):
             for table in tables:
                 # There can be 3 tables, the actions table, an ARN table, and a condition key table
                 # Example: https://docs.aws.amazon.com/IAM/latest/UserGuide/list_awssecuritytokenservice.html
-                if "<th> Actions </th>" not in [
-                    chomp(str(x)) for x in table.find_all("th")
-                ]:
+                if not header_matches("actions", table) or not header_matches("description", table):
                     continue
 
                 rows = table.find_all("tr")
@@ -214,12 +230,9 @@ def create_database(destination_directory, access_level_overrides_file):
                         continue
 
                     if len(cells) != 6:
-                        # Sometimes the privilege might span multiple rows.
-                        # Example: amazonroute53-DisassociateVPCFromHostedZone
-                        # We should be handling this, but if we are not, then bail
-                        raise Exception(
-                            "Unexpected format in {}: {}".format(prefix, row)
-                        )
+                        # Sometimes the privilege contains Scenarios, and I don't know how to handle this
+                        # raise Exception("Unexpected format in {}: {}".format(prefix, row))
+                        break
 
                     # See if this cell spans multiple rows
                     rowspan = 1
@@ -261,7 +274,7 @@ def create_database(destination_directory, access_level_overrides_file):
                     #         access_level = access_level
                     # else:
                     #     access_level = access_level
-                    resource_types = []
+                    resource_types = {}
                     resource_cell = 3
 
                     while rowspan > 0:
@@ -287,13 +300,18 @@ def create_database(destination_directory, access_level_overrides_file):
                                     action_element
                                 ) in dependent_actions_element.find_all("p"):
                                     dependent_actions.append(chomp(action_element.text))
-                            resource_types.append(
-                                {
-                                    "resource_type": resource_type,
-                                    "condition_keys": condition_keys,
-                                    "dependent_actions": dependent_actions,
-                                }
-                            )
+                            if "*" in resource_type:
+                                required = True
+                                resource_type = resource_type.strip("*")
+                            else:
+                                required = False
+
+                            resource_types[resource_type] = {
+                                "resource_type": resource_type,
+                                "required": required,
+                                "condition_keys": condition_keys,
+                                "dependent_actions": dependent_actions,
+                            }
                         rowspan -= 1
                         if rowspan > 0:
                             row_number += 1
@@ -311,14 +329,12 @@ def create_database(destination_directory, access_level_overrides_file):
                         "resource_types": resource_types,
                     }
 
-                    service_schema["privileges"].append(privilege_schema)
+                    service_schema["privileges"][priv] = privilege_schema
                     row_number += 1
 
             # Get resource table
             for table in tables:
-                if "<th> Resource Types </th>" not in [
-                    chomp(str(x)) for x in table.find_all("th")
-                ]:
+                if not header_matches("resource types", table) or not header_matches("arn", table):
                     continue
 
                 rows = table.find_all("tr")
@@ -343,17 +359,15 @@ def create_database(destination_directory, access_level_overrides_file):
                     for condition in cells[2].find_all("p"):
                         conditions.append(chomp(condition.text))
 
-                    service_schema["resources"].append(
-                        {"resource": resource, "arn": arn, "condition_keys": conditions}
-                    )
+                    service_schema["resources"][resource] = {
+                        "resource": resource,
+                        "arn": arn,
+                        "condition_keys": conditions
+                    }
 
             # Get condition keys table
             for table in tables:
-                if "<th> Condition Keys </th>" not in [
-                    chomp(str(x)) for x in table.find_all("th")
-                ] or "<th> Type </th>" not in [
-                    chomp(str(x)) for x in table.find_all("th")
-                ]:
+                if not (header_matches("<th> condition keys </th>", table) and header_matches("<th> type </th>", table)):
                     continue
 
                 rows = table.find_all("tr")
@@ -375,16 +389,18 @@ def create_database(destination_directory, access_level_overrides_file):
                     description = chomp(cells[1].text)
                     value_type = chomp(cells[2].text)
 
-                    service_schema["conditions"].append(
-                        {
-                            "condition": condition,
-                            "description": description,
-                            "type": value_type,
-                        }
-                    )
-            schema.append(service_schema)
+                    service_schema["conditions"][condition] = {
+                        "condition": condition,
+                        "description": description,
+                        "type": value_type,
+                    }
+            this_service_schema = {
+                service_prefix: service_schema
+            }
+            schema.update(this_service_schema)
+            # schema.append(service_schema)
 
-    schema.sort(key=lambda x: x["prefix"])
+    # schema.sort(key=lambda x: x["prefix"])
     iam_definition_file = os.path.join(destination_directory, "iam-definition.json")
     with open(iam_definition_file, "w") as file:
         json.dump(schema, file, indent=4)
